/******************************************************************************/
/*                                                                            */
/*        Project: Boona Taxi                                                 */
/*          Title: Frame counter header file for WIN32 and IRIX               */
/*        Version: 0.31                                                       */
/*                                                                            */
/*        Started: 20/09/2000                                                 */
/*   Last Updated: 19/11/2000                                                 */
/*                                                                            */
/*        Authors: Michael Bonnington, Rob Bateman                            */
/*                                                                            */
/******************************************************************************/

/* frametime.h: A high res frame counter for WIN32 and IRIX. These functions
 *              allow you to create accurate movement by adjusting the frame
 *              rate to compensate for the speed of the computer.
 *
 *              The functions for WIN32 are contained in 'frametime_win.cpp',
 *              and for IRIX in 'frametime_irix.cpp'.
 *
 * In order to keep the gameplay speed consistent on a variety of speeds of
 * processor, a frame counter is required. In the simplest terms, this is a
 * function which is called every cycle of the main loop which measures how long
 * it takes to process each frame. The result of this is stored in a variable
 * named frametime. Now any movement within the game is multiplied by frametime
 * (multiplied or divided by some arbitrary amount) and the result is that on
 * slow processors, frames are skipped to keep the speed up and the movement
 * appears choppy; and on fast processors more frames are drawn and the game
 * runs smoothly.
 *
 * To make use of these functions, you first need to initialise the frame
 * counter within the program. Put something like the following piece of code in
 * your init() function (if you are using GLUT):
 *
 *    if(!initFrameCounter())
 *    {
 *       printf("ERROR: Unable to initialise frame counter.\n\n");
 *       exit(0);
 *    }
 *
 * To calculate the frametime, call getFrametime(). Now multiply any movement
 * by the returned value to get consistent movement. To find out the number
 * frames per second (fps) simply do 1/getFrametime().
 */

#ifndef FRAMETIME_H
#define FRAMETIME_H

/* Function prototypes =======================================================*/
extern int initFrameCounter(void);
extern double ftime(void);
extern double getFrametime(void);

#endif
