/******************************************************************************/
/*                                                                            */
/*        Project: Puzzle Ruckus                                              */
/*          Title: Main program body                                          */
/*        Version: 0.2                                                        */
/*                                                                            */
/*        Started: 10/01/2001                                                 */
/*   Last Updated: 02/02/2001                                                 */
/*                                                                            */
/*        Authors: Martin Johnson, Michael Bonnington                         */
/*                                                                            */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include "./win32.h"
#include "./tetris.h"
#include "./font.h"
#include "./frametime.h"

#include "./tga.h"
typedef unsigned int glTexObj;
typedef unsigned int glDisplayList;
glTexObj tex1;
glDisplayList fighter1, fighter2, portrait1, portrait2, background;

/* Constants =================================================================*/
#define TITLES 1
#define GAME 2
#define PAUSED 3
#define FINISH 4
#define SELECT 5

/* Function prototypes =======================================================*/
void init(void);
void changeGameMode(int);

void drawGrid(int, int, bool);
void drawBlock(int, int, bool);
void drawStatus(void);
void animate(double);

/* GLUT callback functions... */
void idleTitles(void);
void displayTitles(void);
void idleGame(void);
void displayGame(void);
void idlePaused(void);
void displayPaused(void);
void idleFinish(void);
void displayFinish(void);
void idleSelect(void);
void displaySelect(void);
void reshape(int, int);
void keyDown(unsigned char, int, int);
void keyUp(unsigned char, int, int);
void specialDown(int, int, int);
void specialUp(int, int, int);

/* Structure definitions =====================================================*/

/* Class definitions =========================================================*/

/* Globals ===================================================================*/
int win_width, win_height;
int gameMode;
int start_level = 0;

char mikes_string[80];

bool flash;

int timer = 0;
double chronos = 0.0;
int fxtimer = 0;
double fxchronos = 0.0;
int injury = 0;
int fallback = 0;
float slowdown = 5.0;

char p1_score_text[10], p1_lines_text[10], p1_level_text[10],
     p2_score_text[10], p2_lines_text[10], p2_level_text[10];

bool keys[128], specials[128],
     p1_key_left, p1_key_right, p1_key_down, p1_key_rot_left, p1_key_rot_right,
     p2_key_left, p2_key_right, p2_key_down, p2_key_rot_left, p2_key_rot_right;

tetris player1, player2;

/* Functions =================================================================*/

/* Initialisation function ===================================================*/
void init(void)
{
   /* Initialise OpenGL/GLUT stuff... */
   glClearColor(0.5, 0.5, 0.5, 0.0);
   glShadeModel(GL_FLAT);
   glEnable(GL_DEPTH);
   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
   glEnable(GL_BLEND);

   glDepthFunc( GL_LEQUAL );
  
      fighter1 = glGenLists(1);
   glNewList( fighter1 , GL_COMPILE );
     glBegin(GL_QUADS);

       glTexCoord2d( 0 , 1 );
       glVertex2f( -199 , -199 );

       glTexCoord2d( 1 , 1 );
       glVertex2f( 200 , -199 );

       glTexCoord2d( 1 , 0 );
       glVertex2f( 200 , 200 );

       glTexCoord2d( 0 , 0 );
       glVertex2f( -199 , 200 );

     glEnd();
   glEndList();

       fighter2 = glGenLists(2);
   glNewList( fighter2 , GL_COMPILE );
     glBegin(GL_QUADS);

       glTexCoord2d( 0 , 1 );
       glVertex2f( 200 , -199 );

       glTexCoord2d( 1 , 1 );
       glVertex2f( -199 , -199 );

       glTexCoord2d( 1 , 0 );
       glVertex2f( -199 , 200 );

       glTexCoord2d( 0 , 0 );
       glVertex2f( 200 , 200 );

     glEnd();
   glEndList();

        background = glGenLists(3);
   glNewList( background , GL_COMPILE );
     glBegin(GL_QUADS);

       glTexCoord2d( 1 , 1 );
       glVertex2f( 320 , -240);

       glTexCoord2d( 0 , 1 );
       glVertex2f( -320 , -240 );

       glTexCoord2d( 0 , 0 );
       glVertex2f( -320, 240 );

       glTexCoord2d( 1 , 0 );
       glVertex2f( 320, 240 );

     glEnd();
   glEndList();

   portrait1 = glGenLists(4);
   glNewList( portrait1 , GL_COMPILE );
     glBegin(GL_QUADS);

       glTexCoord2d( 0 , 1 );
       glVertex2f( -160 , -240 );

       glTexCoord2d( 1 , 1 );
       glVertex2f( 170 , -240 );

       glTexCoord2d( 1 , 0 );
       glVertex2f( 150 , 240 );

       glTexCoord2d( 0 , 0 );
       glVertex2f( -160 , 240 );

     glEnd();
   glEndList();

       portrait2 = glGenLists(5);
   glNewList( portrait2 , GL_COMPILE );
     glBegin(GL_QUADS);

       glTexCoord2d( 0 , 1 );
       glVertex2f( 160 , -240 );

       glTexCoord2d( 1 , 1 );
       glVertex2f( -150 , -240 );

       glTexCoord2d( 1 , 0 );
       glVertex2f( -170 , 240 );

       glTexCoord2d( 0 , 0 );
       glVertex2f( 160 , 240 );

     glEnd();
   glEndList();

   loadTGA( "titlescreen.tga" , 49, 1 );
   loadTGA( "background.tga" , 50, 1 );
   loadTGA( "backgroundcharge.tga" , 51, 1 );

   loadTGA( "earth1.tga" , 10, 1 );
   loadTGA( "earth2.tga" , 11, 1 );
   loadTGA( "earth3.tga" , 12, 1 );
   loadTGA( "water1.tga" , 20, 1 );
   loadTGA( "water2.tga" , 21, 1 );
   loadTGA( "water3.tga" , 22, 1 );
   loadTGA( "fire1.tga" , 30, 1 );
   loadTGA( "fire2.tga" , 31, 1 );
   loadTGA( "fire3.tga" , 32, 1 );

   loadTGA( "charge1.tga" , 45, 1 );
   loadTGA( "charge2.tga" , 46, 1 );
   loadTGA( "charge3.tga" ,47, 1 );
   
   loadTGA( "shiroportrait.tga" , 70, 1 );
   loadTGA( "rebekhaportrait.tga" , 71, 1 );
   loadTGA( "leanderportrait.tga" , 72, 1 );

   loadTGA( "shirostand1.tga" , 100, 1 );
   loadTGA( "shirostand2.tga" , 101, 1 );
   loadTGA( "shirostand3.tga" , 102, 1 );
   loadTGA( "shirostand4.tga" , 103, 1 );
   loadTGA( "shirocombo1.tga" , 104, 1 );
   loadTGA( "shirocombo2.tga" , 105, 1 );
   loadTGA( "shirocombo3.tga" , 106, 1 );
   loadTGA( "shirocombo4.tga" , 107, 1 );
   loadTGA( "shirospecial1.tga" , 108, 1 );
   loadTGA( "shirospecial2.tga" , 109, 1 );
   loadTGA( "shirospecial3.tga" , 110, 1 );
   loadTGA( "shirospecial4.tga" , 111, 1 );
   loadTGA( "shirosuper1.tga" , 112, 1 );
   loadTGA( "shirosuper2.tga" , 113, 1 );
   loadTGA( "shirosuper3.tga" , 114, 1 );
   loadTGA( "shirosuper4.tga" , 115, 1 );
   loadTGA( "shirosuper5.tga" , 116, 1 );
   loadTGA( "shirosuper6.tga" , 117, 1 );
   loadTGA( "shirotaunt1.tga" , 118, 1 );
   loadTGA( "shirotaunt2.tga" , 119, 1 );
   loadTGA( "shirotaunt3.tga" , 120, 1 );
   loadTGA( "shirohit1.tga" , 121, 1 );
   loadTGA( "shirohit2.tga" , 122, 1 );
   loadTGA( "shirohit3.tga" , 123, 1 );
   loadTGA( "shirodead.tga" , 124, 1 );


   loadTGA( "rebekhastand1.tga" , 125, 1 );
   loadTGA( "rebekhastand2.tga" , 126, 1 );
   loadTGA( "rebekhastand3.tga" , 127, 1 );
   loadTGA( "rebekhastand4.tga" , 128, 1 );
   loadTGA( "rebekhacombo1.tga" , 129, 1 );
   loadTGA( "rebekhacombo2.tga" , 130, 1 );
   loadTGA( "rebekhacombo3.tga" , 131, 1 );
   loadTGA( "rebekhacombo4.tga" , 132, 1 );
   loadTGA( "rebekhaspecial1.tga" , 133, 1 );
   loadTGA( "rebekhaspecial2.tga" , 134, 1 );
   loadTGA( "rebekhaspecial3.tga" , 135, 1 );
   loadTGA( "rebekhaspecial4.tga" , 136, 1 );
   loadTGA( "rebekhasuper1.tga" , 137, 1 );
   loadTGA( "rebekhasuper2.tga" , 138, 1 );
   loadTGA( "rebekhasuper3.tga" , 139, 1 );
   loadTGA( "rebekhasuper4.tga" , 140, 1 );
   loadTGA( "rebekhasuper5.tga" , 141, 1 );
   loadTGA( "rebekhasuper6.tga" , 142, 1 );
   loadTGA( "rebekhataunt1.tga" , 143, 1 );
   loadTGA( "rebekhataunt2.tga" , 144, 1 );
   loadTGA( "rebekhataunt3.tga" , 145, 1 );
   loadTGA( "rebekhahit1.tga" , 146, 1 );
   loadTGA( "rebekhahit2.tga" , 147, 1 );
   loadTGA( "rebekhahit3.tga" , 148, 1 );
   loadTGA( "rebekhadead.tga" , 149, 1 );

   loadTGA( "leanderstand1.tga" , 150, 1 );
   loadTGA( "leanderstand2.tga" , 151, 1 );
   loadTGA( "leanderstand3.tga" , 152, 1 );
   loadTGA( "leanderstand4.tga" , 153, 1 );
   loadTGA( "leandercombo1.tga" , 154, 1 );
   loadTGA( "leandercombo2.tga" , 155, 1 );
   loadTGA( "leandercombo3.tga" , 156, 1 );
   loadTGA( "leandercombo4.tga" , 157, 1 );
   loadTGA( "leanderspecial1.tga" , 158, 1 );
   loadTGA( "leanderspecial2.tga" , 159, 1 );
   loadTGA( "leanderspecial3.tga" , 160, 1 );
   loadTGA( "leanderspecial4.tga" , 161, 1 );
   loadTGA( "leandersuper1.tga" , 162, 1 );
   loadTGA( "leandersuper2.tga" , 163, 1 );
   loadTGA( "leandersuper3.tga" , 164, 1 );
   loadTGA( "leandersuper4.tga" , 165, 1 );
   loadTGA( "leandersuper5.tga" , 166, 1 );
   loadTGA( "leandersuper6.tga" , 167, 1 );
   loadTGA( "leandertaunt1.tga" , 168, 1 );
   loadTGA( "leandertaunt2.tga" , 169, 1 );
   loadTGA( "leandertaunt3.tga" , 170, 1 );
   loadTGA( "leanderhit1.tga" , 171, 1 );
   loadTGA( "leanderhit2.tga" , 172, 1 );
   loadTGA( "leanderhit3.tga" , 173, 1 );
   loadTGA( "leanderdead.tga" , 174, 1 );
   
   glutIgnoreKeyRepeat(1);
   glutSetCursor(GLUT_CURSOR_NONE);

   makeRasterFont();   //initialise font

   /* Initialise frame time function... */
   if(!initFrameCounter())
   {
      printf("ERROR: Unable to initialise frame counter.\n\n");
      exit(0);
   }

    glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE , GL_REPLACE);


   //initTetris();   //initialise Tetris
}

/* Function to change current game mode ======================================*/
void changeGameMode(int mode)
{
   switch(mode)
   {
      case TITLES:
         player1.display_next_block = false;
         player2.display_next_block = false;
         gameMode = TITLES;
         glutIdleFunc(idleTitles);
         glutDisplayFunc(displayTitles);
         break;

      case GAME:
         player1.display_next_block = true;
         player2.display_next_block = true;
         gameMode = GAME;
         glutIdleFunc(idleGame);
         glutDisplayFunc(displayGame);
         break;

      case PAUSED:
         player1.display_next_block = true;
         player2.display_next_block = true;
         gameMode = PAUSED;
         glutIdleFunc(idlePaused);
         glutDisplayFunc(displayPaused);
         break;

      case FINISH:
         gameMode = FINISH;
         glutIdleFunc(idleFinish);
         glutDisplayFunc(displayFinish);
         break;

      case SELECT:
         gameMode = SELECT;
         glutIdleFunc(idleSelect);
         glutDisplayFunc(displaySelect);
         break;

      default:
         break;
   }
}



/* Draw grid =================================================================*/
void drawGrid(int x, int y, bool solid)
{
   glPushMatrix();
    if(solid)
    {
       glColor3f(0.8, 0.6, 1.0);
       glBegin(GL_QUADS);
        glVertex2d(x-5, y-5);
        glVertex2d(x-5, y+5);
        glVertex2d(x+5, y+5);
        glVertex2d(x+5, y-5);
       glEnd();
       glColor3f(0.0, 0.0, 0.0);
    }
    else
    {
       glColor3f(0.7, 0.7, 0.7);
    }
    glBegin(GL_LINE_LOOP);
     glVertex2d(x-5, y-5);
     glVertex2d(x-5, y+5);
     glVertex2d(x+4, y+5);
     glVertex2d(x+4, y-5);
    glEnd();
   glPopMatrix();
}

/* Draw block ================================================================*/
void drawBlock(int x, int y, bool solid)
{
   glPushMatrix();
    if(solid)
    {
       glColor3f(0.0, 0.0, 0.0);
       glBegin(GL_QUADS);
        glVertex2d(x-5, y-5);
        glVertex2d(x-5, y+5);
        glVertex2d(x+5, y+5);
        glVertex2d(x+5, y-5);
       glEnd();
       glColor3f(1.0, 0.8, 0.2);
       glBegin(GL_QUADS);
        glVertex2d(x-4, y-4);
        glVertex2d(x-4, y+4);
        glVertex2d(x+4, y+4);
        glVertex2d(x+4, y-4);
       glEnd();
    }
   glPopMatrix();
}

/* Draw status display =======================================================*/
void drawStatus(void)
{
   int x, y;
   bool p1_next_block[4][4], p2_next_block[4][4];

   /* Player 1... */
   smallFont("Score", 16, win_height-44, 1.0, 1.0, 1.0);
   smallFont(p1_score_text, 16, win_height-32, 0.0, 0.0, 0.0);

   smallFont("Lines", 64, win_height-44, 1.0, 1.0, 1.0);
   smallFont(p1_lines_text, 64, win_height-32, 0.0, 0.0, 0.0);

   smallFont("Level", 104, win_height-44, 1.0, 1.0, 1.0);
   smallFont(p1_level_text, 104, win_height-32, 0.0, 0.0, 0.0);

   smallFont("Next", 144, win_height-44, 1.0, 1.0, 1.0);
   if(player1.display_next_block)
   {
      for(y=0; y<4; y++)
      {
         for(x=0; x<4; x++)
         {
            p1_next_block[x][y] = player1.getNextBlock(x, y);

            if(p1_next_block[x][y])
            {
               glPushMatrix();
               glTranslated(148, win_height-40, 0);
                glBegin(GL_QUADS);
                 glColor3f(0.0, 0.0, 0.0);
                 glVertex2d((x*8)-4, (y*8)-4);
                 glVertex2d((x*8)-4, (y*8)+3);
                 glVertex2d((x*8)+3, (y*8)+3);
                 glVertex2d((x*8)+3, (y*8)-4);
                 glColor3f(1.0, 0.8, 0.2);
                 glVertex2d((x*8)-3, (y*8)-3);
                 glVertex2d((x*8)-3, (y*8)+2);
                 glVertex2d((x*8)+2, (y*8)+2);
                 glVertex2d((x*8)+2, (y*8)-3);
                glEnd();
               glPopMatrix();
            }
         }
      }
   }

   /* Player 2... */
   smallFont("Score", win_width-176, win_height-44, 1.0, 1.0, 1.0);
   smallFont(p2_score_text, win_width-176, win_height-32, 0.0, 0.0, 0.0);

   smallFont("Lines", win_width-128, win_height-44, 1.0, 1.0, 1.0);
   smallFont(p2_lines_text, win_width-128, win_height-32, 0.0, 0.0, 0.0);

   smallFont("Level", win_width-88, win_height-44, 1.0, 1.0, 1.0);
   smallFont(p2_level_text, win_width-88, win_height-32, 0.0, 0.0, 0.0);

   smallFont("Next", win_width-48, win_height-44, 1.0, 1.0, 1.0);
   if(player2.display_next_block)
   {
      for(y=0; y<4; y++)
      {
         for(x=0; x<4; x++)
         {
            p2_next_block[x][y] = player2.getNextBlock(x, y);

            if(p2_next_block[x][y])
            {
               glPushMatrix();
               glTranslated(win_width-44, win_height-40, 0);
                glBegin(GL_QUADS);
                 glColor3f(0.0, 0.0, 0.0);
                 glVertex2d((x*8)-4, (y*8)-4);
                 glVertex2d((x*8)-4, (y*8)+3);
                 glVertex2d((x*8)+3, (y*8)+3);
                 glVertex2d((x*8)+3, (y*8)-4);
                 glColor3f(1.0, 0.8, 0.2);
                 glVertex2d((x*8)-3, (y*8)-3);
                 glVertex2d((x*8)-3, (y*8)+2);
                 glVertex2d((x*8)+2, (y*8)+2);
                 glVertex2d((x*8)+2, (y*8)-3);
                glEnd();
               glPopMatrix();
            }
         }
      }
   }

   /* Energy bars... */
   glPushMatrix();
    glBegin(GL_QUADS);
     glColor3f(0.0, 0.0, 0.0);
     glVertex2d((win_width/2)-302, 18);
     glVertex2d((win_width/2)-302, 46);
     glVertex2d((win_width/2)+302, 46);
     glVertex2d((win_width/2)+302, 18);
     glColor3f(1.0, 1.0, 1.0);
     glVertex2d((win_width/2)-300, 20);
     glVertex2d((win_width/2)-300, 44);
     glVertex2d((win_width/2)-16, 44);
     glVertex2d((win_width/2)-16, 20);
     glVertex2d((win_width/2)+16, 20);
     glVertex2d((win_width/2)+16, 44);
     glVertex2d((win_width/2)+300, 44);
     glVertex2d((win_width/2)+300, 20);
     glColor3f(0.0, 0.0, 0.0);
     glVertex2d((win_width/2)-298, 22);
     glVertex2d((win_width/2)-298, 42);
     glVertex2d((win_width/2)-16, 42);
     glVertex2d((win_width/2)-16, 22);
     glVertex2d((win_width/2)+16, 22);
     glVertex2d((win_width/2)+16, 42);
     glVertex2d((win_width/2)+298, 42);
     glVertex2d((win_width/2)+298, 22);
     glColor3f(0.8, 0.0, 0.0);
     glVertex2d((win_width/2)-296, 24);
     glVertex2d((win_width/2)-296, 40);
     glVertex2d((win_width/2)-16, 40);
     glVertex2d((win_width/2)-16, 24);
     glVertex2d((win_width/2)+16, 24);
     glVertex2d((win_width/2)+16, 40);
     glVertex2d((win_width/2)+296, 40);
     glVertex2d((win_width/2)+296, 24);
     glColor3f(1.0, (float)player2.opponent_energy/100, 0.0);
     glVertex2d((win_width/2)-16-player2.opponent_energy*2.8, 24);
     glVertex2d((win_width/2)-16-player2.opponent_energy*2.8, 40);
     glVertex2d((win_width/2)-16, 40);
     glVertex2d((win_width/2)-16, 24);
     glColor3f(1.0, (float)player1.opponent_energy/100, 0.0);
     glVertex2d((win_width/2)+16, 24);
     glVertex2d((win_width/2)+16, 40);
     glVertex2d((win_width/2)+16+player1.opponent_energy*2.8, 40);
     glVertex2d((win_width/2)+16+player1.opponent_energy*2.8, 24);
    glEnd();

   //draw player 1 name...
   switch(player1.character_offset)
   {
      case 0:
         largeOutlineFont("SHIRO",   16, 80, 1.0, 1.0, 1.0);
         break;

      case 1:
         largeOutlineFont("REBEKHA", 16, 80, 1.0, 1.0, 1.0);
         break;

      case 2:
         largeOutlineFont("LEANDER", 16, 80, 1.0, 1.0, 1.0);
         break;

      default:
         break;
   }
   //draw player 2 name...
   switch(player2.character_offset)
   {
      case 0:
         largeOutlineFont("SHIRO",    win_width-96, 80, 1.0, 1.0, 1.0);
         break;

      case 1:
         largeOutlineFont("REBEKHA", win_width-128, 80, 1.0, 1.0, 1.0);
         break;

      case 2:
         largeOutlineFont("LEANDER", win_width-128, 80, 1.0, 1.0, 1.0);
         break;

      default:
         break;
   }



    largeOutlineFont("KO", (win_width/2)-16, 48, 1.0, 1.0, 1.0);
   glPopMatrix();

   /* Power bars... */
   glPushMatrix();
    glBegin(GL_QUADS);
     glColor3f(0.0, 0.0, 0.0);
     glVertex2d((win_width/2)-122, win_height-46);
     glVertex2d((win_width/2)-122, win_height-18);
     glVertex2d((win_width/2)+122, win_height-18);
     glVertex2d((win_width/2)+122, win_height-46);
     glColor3f(1.0, 1.0, 1.0);
     glVertex2d((win_width/2)-120, win_height-44);
     glVertex2d((win_width/2)-120, win_height-20);
     glVertex2d((win_width/2)-16, win_height-20);
     glVertex2d((win_width/2)-16, win_height-44);
     glVertex2d((win_width/2)+16, win_height-44);
     glVertex2d((win_width/2)+16, win_height-20);
     glVertex2d((win_width/2)+120, win_height-20);
     glVertex2d((win_width/2)+120, win_height-44);
     glColor3f(0.0, 0.0, 0.0);
     glVertex2d((win_width/2)-118, win_height-42);
     glVertex2d((win_width/2)-118, win_height-22);
     glVertex2d((win_width/2)-16, win_height-22);
     glVertex2d((win_width/2)-16, win_height-42);
     glVertex2d((win_width/2)+16, win_height-42);
     glVertex2d((win_width/2)+16, win_height-22);
     glVertex2d((win_width/2)+118, win_height-22);
     glVertex2d((win_width/2)+118, win_height-42);
     glColor3f(0.0, 0.6, 0.3);
     glVertex2d((win_width/2)-116, win_height-40);
     glVertex2d((win_width/2)-116, win_height-24);
     glVertex2d((win_width/2)-16, win_height-24);
     glVertex2d((win_width/2)-16, win_height-40);
     glVertex2d((win_width/2)+16, win_height-40);
     glVertex2d((win_width/2)+16, win_height-24);
     glVertex2d((win_width/2)+116, win_height-24);
     glVertex2d((win_width/2)+116, win_height-40);
     glColor3f(0.0, 1.0, (float)player1.power/100);
     glVertex2d((win_width/2)-16-player1.power, win_height-40);
     glVertex2d((win_width/2)-16-player1.power, win_height-24);
     glVertex2d((win_width/2)-16, win_height-24);
     glVertex2d((win_width/2)-16, win_height-40);
     glColor3f(0.0, 1.0, (float)player2.power/100);
     glVertex2d((win_width/2)+16, win_height-40);
     glVertex2d((win_width/2)+16, win_height-24);
     glVertex2d((win_width/2)+16+player2.power, win_height-24);
     glVertex2d((win_width/2)+16+player2.power, win_height-40);
    glEnd();

    if(player1.power >= 90)
       largeOutlineFont("MAX", (win_width/2)-96, win_height-16, flash, 1.0, 1.0);
    if(player2.power >= 90)
       largeOutlineFont("MAX", (win_width/2)+48, win_height-16, flash, 1.0, 1.0);

    largeOutlineFont("CHI", (win_width/2)-24, win_height-16, 1.0, 1.0, 1.0);
   glPopMatrix();
}



/* Idle function (Titles) ====================================================*/
void idleTitles(void)
{
   double frametime = getFrametime();

   player1.character_offset = 0;
   player2.character_offset = 0;

   if(specials[GLUT_KEY_LEFT])   //decrease level
   {
      if(start_level > 0) start_level--;
      else start_level = 9;
      specials[GLUT_KEY_LEFT] = false;
   }

   if(specials[GLUT_KEY_RIGHT])   //increase level
   {
      if(start_level < 9) start_level++;
      else start_level = 0;
      specials[GLUT_KEY_RIGHT] = false;
   }

   if(keys[27])   //quit
   {
      exit(0);
      keys[27] = false;
   }

   if(keys[13])   //begin game
   {
      changeGameMode(SELECT);
      keys[13] = false;
   }

   sprintf(mikes_string, "%d", start_level);

   glutPostRedisplay();
}

/* Display function (Titles) =================================================*/
void displayTitles(void)
{
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   glLoadIdentity();

   glPushMatrix();
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,49);            //  BACKGROUND
    glTranslatef(320, 240, 0);
    glCallList(background);
   glPopMatrix();

   glDisable(GL_TEXTURE_2D);

   smallFont("Select a level with LEFT or RIGHT.", 16, 32, 1.0, 1.0, 0.6);
   smallFont("ENTER starts new game, ESC exits.", 16, 48, 1.0, 1.0, 0.6);
   largeFont("Level < >", 16, 96, 0.6, 0.4, 1.0);
   largeOutlineFont(mikes_string, 128, 96, 1.0, 1.0, 0.6);

   largeOutlineFont("U R B A N   F I G H T E R",
                    (win_width/2)-200, win_height-32, 1.0, 1.0, 1.0);

   glutSwapBuffers();
}

/* Idle function (Game) ======================================================*/
void idleGame(void)
{  
   fallback = 0;

   double frametime = getFrametime();

   animate(frametime);

   srand(time(NULL));

   /* Player 1 controls... */
   if(!player1.game_over)
   {
      if(p1_key_left)   //move left
      {
         player1.moveBlock(-1);
         p1_key_left = false;
      }

      if(p1_key_right)   //move right
      {
         player1.moveBlock(1);
         p1_key_right = false;
      }

      if(p1_key_down)   //drop
      {
         player1.dropBlock(frametime);
      }

      if(p1_key_rot_left)   //rotate anti-clockwise
      {
         player1.rotateBlock(0);
         p1_key_rot_left = false;
      }

      if(p1_key_rot_right)   //rotate clockwise
      {
         player1.rotateBlock(1);
         p1_key_rot_right = false;
      }
   }
   else changeGameMode(TITLES);

   /* Player 2 controls... */
   if(!player2.game_over)
   {
      if(p2_key_left)   //move left
      {
         player2.moveBlock(-1);
         p2_key_left = false;
      }

      if(p2_key_right)   //move right
      {
         player2.moveBlock(1);
         p2_key_right = false;
      }

      if(p2_key_down)   //drop
      {
         player2.dropBlock(frametime);
      }

      if(p2_key_rot_left)   //rotate anti-clockwise
      {
         player2.rotateBlock(0);
         p2_key_rot_left = false;
      }

      if(p2_key_rot_right)   //rotate clockwise
      {
         player2.rotateBlock(1);
         p2_key_rot_right = false;
      }
   }
   else changeGameMode(TITLES);

   if(specials[GLUT_KEY_F1])   //test p1 supercombo
   {
      player1.attack = 4;
      specials[GLUT_KEY_F1] = false;
   }

   if(specials[GLUT_KEY_F2])   //test p2 supercombo
   {
      player2.attack = 4;
      specials[GLUT_KEY_F2] = false;
   }

   if(player1.update(p1_key_down, frametime))
   {
      p1_key_left = false;
      p1_key_right = false;
      //p1_key_down = false;
   }

   if(player2.update(p2_key_down, frametime))
   {
      p2_key_left = false;
      p2_key_right = false;
      //p2_key_down = false;
   }

   if(keys[27])   //quit
   {
      changeGameMode(TITLES);
      keys[27] = false;
   }

   sprintf(p1_score_text, "%d", player1.score);
   sprintf(p1_lines_text, "%d", player1.lines);
   sprintf(p1_level_text, "%d", player1.level);
   sprintf(p2_score_text, "%d", player2.score);
   sprintf(p2_lines_text, "%d", player2.lines);
   sprintf(p2_level_text, "%d", player2.level);

   if(player1.attack != 0 || player2.attack != 0)   //pause
   {
      fxchronos = 0.3;
      changeGameMode(PAUSED);
      chronos = 0.0;
   }

   glutPostRedisplay();
}

/* Display function (Game) ===================================================*/
void displayGame(void)
{
   int x, y;

   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   glLoadIdentity();

    glPushMatrix();
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,50);            //  BACKGROUND
    glTranslatef(320, 240, 0);
    glCallList(background);
    glPopMatrix();
 
    glPushMatrix();
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,100 + player1.character_offset*25 + timer);            //PLAYER 1 CHARACTER
    glTranslatef(260, 200, 0);
    glCallList(fighter1);
    glPopMatrix();

    glPushMatrix();
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,100 + player2.character_offset*25 + timer);            //PLAYER 2 CHARACTER
    glTranslatef(375, 200, 0);
    glCallList(fighter2);
    glPopMatrix();


   glDisable(GL_TEXTURE_2D);

   drawStatus();

   /* Player 1 grid... */
   glPushMatrix();
    glTranslated(8, 96, 0);

    /* Draw grid (p1)... */
    for(y=0; y<24; y++)
    {
       for(x=1; x<11; x++)
       {
          drawGrid(x*10, y*10, player1.grid[x][y]);
       }
    }

    /* Draw current block (p1)... */
    for(y=0; y<4; y++)
    {
       for(x=0; x<4; x++)
       {
          drawBlock((x+(int)player1.pos_x)*10, (y+(int)player1.pos_y)*10, player1.current_block[x][y]);
       }
    }
   glPopMatrix();

   /* Player 2 grid... */
   glPushMatrix();
    glTranslated(win_width-120, 96, 0);

    /* Draw grid (p2)... */
    for(y=0; y<24; y++)
    {
       for(x=1; x<11; x++)
       {
          drawGrid(x*10, y*10, player2.grid[x][y]);
       }
    }

    /* Draw current block (p2)... */
    for(y=0; y<4; y++)
    {
       for(x=0; x<4; x++)
       {
          drawBlock((x+(int)player2.pos_x)*10, (y+(int)player2.pos_y)*10, player2.current_block[x][y]);
       }
    }
   glPopMatrix();
 
   glutSwapBuffers();
}



/* Idle function (Paused) ====================================================*/
void idlePaused(void)
{
   double frametime = getFrametime();

   animate(frametime);

   if(fxchronos != 0.0)
   {
   fxchronos += frametime*7.0;
   fxtimer = (int)floor(fxchronos);
    if(fxchronos >= 2.0)
    {
    fxchronos = 0.0;
    }
   }

   if((injury == 23)||(injury == 24)) fallback = 100;
   
   else fallback = 0;
      
   if(player1.opponent_energy <= 0 || player2.opponent_energy <= 0)
   {
      player1.game_over = true;
      player2.game_over = true;
      fxtimer = 0;
      fxchronos = 0.3;
      changeGameMode(FINISH);
   }

   if(player1.attack == 0 && player2.attack == 0)   //unpause
   {
      timer = 0;
      injury = 0;
      fallback = 0;
      changeGameMode(GAME);
   }

   glutPostRedisplay();
}

/* Display function (Paused) =================================================*/
void displayPaused(void)
{

   int x, y; 

   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   glLoadIdentity();

   if((player1.attack == 4 || player2.attack == 4) && (chronos < 1.0))
    { 
    glPushMatrix();
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,51);            //  CHARGE BACKGROUND
    glTranslatef(320, 240, 0);
    glCallList(background);
    glPopMatrix();
    }
   
   else
    { 
    glPushMatrix();
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,50);            //  BACKGROUND
    glTranslatef(320, 240, 0);
    glCallList(background);
    glPopMatrix();
    }

   glDisable(GL_TEXTURE_2D);

   /* Player 1 grid... */
   glPushMatrix();
    glTranslated(8, 96, 0);

    /* Draw grid (p1)... */
    for(y=0; y<24; y++)
    {
       for(x=1; x<11; x++)
       {
          drawGrid(x*10, y*10, player1.grid[x][y]);
       }
    }

    /* Draw current block (p1)... */
    for(y=0; y<4; y++)
    {
       for(x=0; x<4; x++)
       {
          drawBlock((x+(int)player1.pos_x)*10, (y+(int)player1.pos_y)*10, player1.current_block[x][y]);
       }
    }
   glPopMatrix();

   /* Player 2 grid... */
   glPushMatrix();
    glTranslated(win_width-120, 96, 0);

    /* Draw grid (p2)... */
    for(y=0; y<24; y++)
    {
       for(x=1; x<11; x++)
       {
          drawGrid(x*10, y*10, player2.grid[x][y]);
       }
    }

    /* Draw current block (p2)... */
    for(y=0; y<4; y++)
    {
       for(x=0; x<4; x++)
       {
          drawBlock((x+(int)player2.pos_x)*10, (y+(int)player2.pos_y)*10, player2.current_block[x][y]);
       }
    }
   glPopMatrix();

   drawStatus();

 if(player2.attack != 0)
 {
  
  glPushMatrix();
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,100 + player1.character_offset*25 + injury);             //PLAYER 1 CHARACTER
    glTranslatef(260 - fallback, 200, 0);
    glCallList(fighter1);
   glPopMatrix();


  glPushMatrix();
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,100 + player2.character_offset*25 + timer);            //PLAYER 2 CHARACTER
    glTranslatef(375, 200, 0);
    glCallList(fighter2);
   glPopMatrix();

 if((player2.attack == 4) && (chronos < 1.0))
   {
   glPushMatrix();
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,45 + fxtimer);            //PLAYER 2 CHARGE
    glTranslatef(375, 200, 0);
    glCallList(fighter2);
   glPopMatrix();
   }  

 }

 if(player1.attack != 0)
 
 {

   glPushMatrix();
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,100 + player2.character_offset*25 + injury);             //PLAYER 2 CHARACTER
    glTranslatef(375 + fallback, 200, 0);
    glCallList(fighter2);
   glPopMatrix();


   glPushMatrix();
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,100 + player1.character_offset*25 + timer);            //PLAYER 1 CHARACTER
    glTranslatef(260, 200, 0);
    glCallList(fighter1);
   glPopMatrix();

  if((player1.attack == 4) && (chronos < 1.0))
   {
  glPushMatrix();
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,45 + fxtimer);            //PLAYER 1 CHARGE
    glTranslatef(260, 200, 0);
    glCallList(fighter1);
   glPopMatrix();
   }

 }

/* Player 1 grid... */
   glPushMatrix();
    glTranslated(8, 40, 0);

    /* Draw grid (p1)... */
    for(y=0; y<24; y++)
    {
       for(x=1; x<11; x++)
       {
          drawGrid(x*10, y*10, player1.grid[x][y]);
       }
    }

    /* Draw current block (p1)... */
    for(y=0; y<4; y++)
    {
       for(x=0; x<4; x++)
       {
          drawBlock((x+(int)player1.pos_x)*10, (y+(int)player1.pos_y)*10, player1.current_block[x][y]);
       }
    }
   glPopMatrix();

   /* Player 2 grid... */
   glPushMatrix();
    glTranslated(win_width-120, 40, 0);

    /* Draw grid (p2)... */
    for(y=0; y<24; y++)
    {
       for(x=1; x<11; x++)
       {
          drawGrid(x*10, y*10, player2.grid[x][y]);
       }
    }

    /* Draw current block (p2)... */
    for(y=0; y<4; y++)
    {
       for(x=0; x<4; x++)
       {
          drawBlock((x+(int)player2.pos_x)*10, (y+(int)player2.pos_y)*10, player2.current_block[x][y]);
       }
    }
   glPopMatrix();   

   glutSwapBuffers();
}

/* Idle function (Finish) ====================================================*/
void idleFinish(void)
{

   double frametime = getFrametime();

   animate(frametime/slowdown);

   if(fxchronos != 0.0)
   {
   fxchronos += frametime*4.0;
   fxtimer = (int)floor(fxchronos);
    if(fxchronos >= 2.0)
    {
    fxchronos = 0.0;
    }
   }

   if((injury == 23)||(injury == 24)) fallback = 100;
      
   if(slowdown > 1.0) slowdown -= frametime*1.5;
 
   if(specials[GLUT_KEY_F4])   //reset
   {
      changeGameMode(GAME);
      player1.reset(start_level);
      player2.reset(start_level);
      slowdown = 5.0;
      fxtimer = 0;
      specials[GLUT_KEY_F4] = false;
   }

   if(keys[27])   //quit
   {
      changeGameMode(TITLES);
      keys[27] = false;
   }


   glutPostRedisplay();
}

/* Display function (Finish) =================================================*/
void displayFinish(void)
{

   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   glLoadIdentity();

    if((player1.attack != 4) && (player2.attack != 4))
    {
    glPushMatrix();
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,50);            //  BACKGROUND
    glTranslatef(320, 240, 0);
    glCallList(background);
    glPopMatrix();
   

   glDisable(GL_TEXTURE_2D);

   //drawStatus();
   }

if(player2.attack != 0)
 {
   if(player2.attack == 4)
    {
   glPushMatrix();
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,(player2.character_offset+1)*10 + fxtimer);            //  PLAYER2 ELEMENTAL FINISH
    glTranslatef(320, 240, 0);
    glCallList(background);
    glPopMatrix();
    }    

  glPushMatrix();
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,100 + player1.character_offset*25 + injury);             //PLAYER 1 CHARACTER
    glTranslatef(260 - fallback, 200, 0);
    glCallList(fighter1);
   glPopMatrix();


  glPushMatrix();
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,100 + player2.character_offset*25 + timer);            //PLAYER 2 CHARACTER
    glTranslatef(375, 200, 0);
    glCallList(fighter2);
   glPopMatrix();

 }

 if(player1.attack != 0)
 
 {
   if(player1.attack == 4)
    {
   glPushMatrix();
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,(player1.character_offset+1)*10 + fxtimer);            //  PLAYER1 ELEMENTAL FINISH
    glTranslatef(320, 240, 0);
    glCallList(background);
    glPopMatrix();
    }

   glPushMatrix();
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,100 + player2.character_offset*25 + injury);             //PLAYER 2 CHARACTER
    glTranslatef(375 + fallback, 200, 0);
    glCallList(fighter2);
   glPopMatrix();


   glPushMatrix();
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,100 + player1.character_offset*25 + timer);            //PLAYER 1 CHARACTER
    glTranslatef(260, 200, 0);
    glCallList(fighter1);
   glPopMatrix();

 }
  
 if((player1.attack == 0) && (player2.attack == 0))
 {
   if(player1.opponent_energy <= 0)
   {

   glPushMatrix();
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,100 + player2.character_offset*25 + 24);             //PLAYER 2 CHARACTER
    glTranslatef(375 + fallback, 200, 0);
    glCallList(fighter2);
   glPopMatrix();


   glPushMatrix();
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,100 + player1.character_offset*25 + timer);            //PLAYER 1 CHARACTER
    glTranslatef(260, 200, 0);
    glCallList(fighter1);
   glPopMatrix();

   }

   else
   {

   glPushMatrix();
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,100 + player1.character_offset*25 + 24);             //PLAYER 1 CHARACTER
    glTranslatef(260 - fallback, 200, 0);
    glCallList(fighter1);
   glPopMatrix();


   glPushMatrix();
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,100 + player2.character_offset*25 + timer);            //PLAYER 2 CHARACTER
    glTranslatef(375, 200, 0);
    glCallList(fighter2);
   glPopMatrix();

   }

 }


   glutSwapBuffers();


}

/* Idle function (Select) ====================================================*/
void idleSelect(void)
{

   double frametime = getFrametime();
 
   if(keys[13])   //reset
   {
      player1.reset(start_level);
      player2.reset(start_level);
      slowdown = 5.0;
      changeGameMode(GAME); 
      keys[13] = false;
   }

   if(player1.character_offset != 0)
   {
    if(p1_key_left)   //move left
      {
         player1.character_offset -= 1;
         p1_key_left = false;
      }
   }

   if(player1.character_offset != 2)
   {
      if(p1_key_right)   //move right
      {
         player1.character_offset += 1;
         p1_key_right = false;
      }
   }
 
   if(player2.character_offset != 0)
   {
    if(p2_key_left)   //move left
      {
         player2.character_offset -= 1;
         p2_key_left = false;
      }
   }

   if(player2.character_offset != 2)
   {
      if(p2_key_right)   //move right
      {
         player2.character_offset += 1;
         p2_key_right = false;
      }
   }

   glutPostRedisplay();
}

/* Display function (Select) =================================================*/
void displaySelect(void)
{

   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   glLoadIdentity();

    glPushMatrix();
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,70 + player1.character_offset);            //  PLAYER 1
    glTranslatef(160, 240, 0);
    glCallList(portrait1);
    glPopMatrix();

    glPushMatrix();
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,70 + player2.character_offset);            //  PLAYER 2
    glTranslatef(480, 240, 0);
    glCallList(portrait2);
    glPopMatrix();


   glDisable(GL_TEXTURE_2D);

   //draw player 1 name and vital stats...
   switch(player1.character_offset)
   {
      case 0:
         largeOutlineFont("SHIRO", 16, win_height-64, 1.0, 0.1, 0.2);
         smallFont("Martial Art: Shin Ken Do (True Fist)", 17, win_height-47, 0.0, 0.0, 0.0);
         smallFont("Martial Art: Shin Ken Do (True Fist)", 16, win_height-48, 1.0, 1.0, 1.0);
         smallFont("Element: Earth", 17, win_height-31, 0.0, 0.0, 0.0);
         smallFont("Element: Earth", 16, win_height-32, 1.0, 1.0, 1.0);
         break;

      case 1:
         largeOutlineFont("REBEKHA", 16, win_height-64, 0.6, 0.8, 1.0);
         smallFont("Martial Art: Tang Soo Do", 17, win_height-47, 0.0, 0.0, 0.0);
         smallFont("Martial Art: Tang Soo Do", 16, win_height-48, 1.0, 1.0, 1.0);
         smallFont("Element: Water", 17, win_height-31, 0.0, 0.0, 0.0);
         smallFont("Element: Water", 16, win_height-32, 1.0, 1.0, 1.0);
         break;

      case 2:
         largeOutlineFont("LEANDER", 16, win_height-64, 1.0, 0.9, 0.0);
         smallFont("Martial Art: Wu Shu", 17, win_height-47, 0.0, 0.0, 0.0);
         smallFont("Martial Art: Wu Shu", 16, win_height-48, 1.0, 1.0, 1.0);
         smallFont("Element: Fire", 17, win_height-31, 0.0, 0.0, 0.0);
         smallFont("Element: Fire", 16, win_height-32, 1.0, 1.0, 1.0);
         break;

      default:
         break;
   }

   //draw player 2 name and vital stats...
   switch(player2.character_offset)
   {
      case 0:
         largeOutlineFont("SHIRO", (win_width/2)+16, win_height-64, 1.0, 0.1, 0.2);
         smallFont("Martial Art: Shin Ken Do (True Fist)", (win_width/2)+17, win_height-47, 0.0, 0.0, 0.0);
         smallFont("Martial Art: Shin Ken Do (True Fist)", (win_width/2)+16, win_height-48, 1.0, 1.0, 1.0);
         smallFont("Element: Earth", (win_width/2)+17, win_height-31, 0.0, 0.0, 0.0);
         smallFont("Element: Earth", (win_width/2)+16, win_height-32, 1.0, 1.0, 1.0);
         break;

      case 1:
         largeOutlineFont("REBEKHA", (win_width/2)+16, win_height-64, 0.6, 0.8, 1.0);
         smallFont("Martial Art: Tang Soo Do", (win_width/2)+17, win_height-47, 0.0, 0.0, 0.0);
         smallFont("Martial Art: Tang Soo Do", (win_width/2)+16, win_height-48, 1.0, 1.0, 1.0);
         smallFont("Element: Water", (win_width/2)+17, win_height-31, 0.0, 0.0, 0.0);
         smallFont("Element: Water", (win_width/2)+16, win_height-32, 1.0, 1.0, 1.0);
         break;

      case 2:
         largeOutlineFont("LEANDER", (win_width/2)+16, win_height-64, 1.0, 0.9, 0.0);
         smallFont("Martial Art: Wu Shu", (win_width/2)+17, win_height-47, 0.0, 0.0, 0.0);
         smallFont("Martial Art: Wu Shu", (win_width/2)+16, win_height-48, 1.0, 1.0, 1.0);
         smallFont("Element: Fire", (win_width/2)+17, win_height-31, 0.0, 0.0, 0.0);
         smallFont("Element: Fire", (win_width/2)+16, win_height-32, 1.0, 1.0, 1.0);
         break;

      default:
         break;
   }

   glEnable(GL_TEXTURE_2D);



   glutSwapBuffers();


}


/* Reshape function ==========================================================*/
void reshape(int w, int h)
{
   if(h == 0) h = 1;   //required to prevent crash when window is minimised
   win_width = w; win_height = h;

   glViewport(0, 0, (GLsizei) w, (GLsizei) h);

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glOrtho(0, w, h, 0, -1, 1);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
}

/* Keyboard function - key is pressed ========================================*/
void keyDown(unsigned char key, int x, int y)
{
   keys[key] = true;

   switch(key)
   {
      case 'a':
      case 'A':
         p1_key_left = true;
         break;

      case 'd':
      case 'D':
         p1_key_right = true;
         break;

      case 's':
      case 'S':
         p1_key_down = true;
         break;

      case 'v':
      case 'V':
      case 'w':
      case 'W':
         p1_key_rot_left = true;
         break;

      case 'b':
      case 'B':
         p1_key_rot_right = true;
         break;

      case 'o':
      case 'O':
         p2_key_rot_left = true;
         break;

      case 'p':
      case 'P':
         p2_key_rot_right = true;
         break;

      default:
         break;
   }
}

/* Keyboard function - key is released =======================================*/
void keyUp(unsigned char key, int x, int y)
{
   keys[key] = false;

   switch(key)
   {
      case 'a':
      case 'A':
         p1_key_left = false;
         break;

      case 'd':
      case 'D':
         p1_key_right = false;
         break;

      case 's':
      case 'S':
         p1_key_down = false;
         break;

      case 'v':
      case 'V':
      case 'w':
      case 'W':
         p1_key_rot_left = false;
         break;

      case 'b':
      case 'B':
         p1_key_rot_right = false;
         break;

      case 'o':
      case 'O':
         p2_key_rot_left = false;
         break;

      case 'p':
      case 'P':
         p2_key_rot_right = false;
         break;

      default:
         break;
   }
}

/* Keyboard function - special key is pressed ================================*/
void specialDown(int key, int x, int y)
{
   specials[key] = true;

   switch(key)
   {
      case GLUT_KEY_LEFT:
         p2_key_left = true;
         break;

      case GLUT_KEY_RIGHT:
         p2_key_right = true;
         break;

      case GLUT_KEY_DOWN:
         p2_key_down = true;
         break;

      case GLUT_KEY_UP:
         p2_key_rot_left = true;
         break;

      default:
         break;
   }
}

/* Keyboard function - special key is released ===============================*/
void specialUp(int key, int x, int y)
{
   specials[key] = false;

   switch(key)
   {
      case GLUT_KEY_LEFT:
         p2_key_left = false;
         break;

      case GLUT_KEY_RIGHT:
         p2_key_right = false;
         break;

      case GLUT_KEY_DOWN:
         p2_key_down = false;
         break;

      case GLUT_KEY_UP:
         p2_key_rot_left = false;
         break;

      default:
         break;
   }
}

void animate(double frametime)
{ 

if(player1.attack > player2.attack)       // player1 attacking
{
 switch(player1.attack)
 {

  case 4:

  switch(player1.character_offset)
  { 

case 2:    // LEANDER

  chronos += frametime*2.0;

  if((chronos >= 1.0) && (chronos < 15.0)) 
  {
  chronos += frametime*9.0;
  }

  if(chronos >= 17.0)
  {
  player1.attack = 0;
  chronos = 0.0;
  }

  if((int)floor(chronos) == 0.0)
  {
  timer = 12;
  }

  if((int)floor(chronos) == 1.0 || (int)floor(chronos) == 4.0 || (int)floor(chronos) == 7.0 || 
     (int)floor(chronos) == 10.0 || (int)floor(chronos) == 13.0 ) 
  {
     timer = 13;
     if(injury != 23)
     {
      player1.opponent_energy -= 5;
      injury = 23;
     }
     
  }

  if((int)floor(chronos) == 2.0 || (int)floor(chronos) == 5.0 || (int)floor(chronos) == 8.0 || 
     (int)floor(chronos) == 11.0 || (int)floor(chronos) == 14.0 ) 
  {
     timer = 14;
     if(injury != 22)
     {
      player1.opponent_energy -= 5;
      injury = 22;
     }

  }

  if((int)floor(chronos) == 3.0 || (int)floor(chronos) == 6.0 || (int)floor(chronos) == 9.0 || 
     (int)floor(chronos) == 12.0 || (int)floor(chronos) == 15.0 ) 
  {
     timer = 15;
     injury = 21;
  }

  


  if(chronos >= 15.0) 
  {
    timer = (int)floor(chronos)+1;
    injury = timer + 7;
  }

  break;

  case 1:    // REBEKHA

  chronos += frametime*2.0;

  if(chronos >= 11.0)
  {
  player1.attack = 0;
  chronos = 0.0;
  }
 
  if((chronos >= 1.0) && (chronos < 10.0)) chronos += frametime*8.0;
 
  if(chronos < 4.0)
  {
  timer = (int)floor(chronos)+12;
   if((chronos >= 1.0) && ((int)floor(chronos) != 2.0) && (injury != timer+8))
   {
   player1.opponent_energy -= 10;
   injury = timer+8;
   }
  }

  if((int)floor(chronos) == 4.0 || (int)floor(chronos) == 6.0 || (int)floor(chronos) == 8.0)
  {
  fallback = 0;
  timer = 14;
  injury = 21;
  }

  if((int)floor(chronos) == 7.0) 
  {
  fallback = 0;
  timer = 15;
   if(injury != 22)
   {
   player1.opponent_energy -= 10;
   injury = 22;
   }
  }

  if((int)floor(chronos) == 5.0 || (int)floor(chronos) == 9.0)
  {
  fallback = 100;
  timer = 16;
   if(injury != 22)
   {
   player1.opponent_energy -= 10;
   injury = 22;
   }
  }

  if(chronos >= 10.0) 
  {
  timer = 17;
  injury = 23;
  }
  
  break;

 
  default:    // CASE 0 - SHIRO

  chronos += frametime*1.0;

  if(chronos >= 3.0)
  {
  chronos += frametime*1.5;
  }

  if(chronos >= 4.0)
  {
  chronos += frametime*1.5;
  }

  if(chronos >= 6.0)
  {
  player1.attack = 0;
  chronos = 0.0;
  }

  timer = (int)floor(chronos)+12;
  if((chronos >= 1.0) && (chronos < 4.0) && (injury != timer+8))
  {
  player1.opponent_energy -= 17;
  injury = timer+8;
  }
  if(chronos >= 4.0)                      injury = 24;
  
  break;

  }

 break;

 case 3:

 chronos += frametime*3.5;
  if(chronos >= 4.0)
  {
   player1.attack = 0;
   chronos = 0.0;
  }
 timer = (int)floor(chronos)+8;
  if((chronos >= 2.0)&&(injury != 23))
  {
  player1.opponent_energy -= 25;
  injury = 23;
  }

 break;


 case 2:

 chronos += frametime*3.3;
  if(chronos >= 4.0)
  {
  player1.attack = 0;
  chronos = 0.0;
  }
 timer = (int)floor(chronos)+4;

 if((injury != timer+17)&&(chronos < 3.0)&&(player1.attack == 2))
  {
  player1.opponent_energy -= 5;
  injury = timer+17;
  }

 break;

 case 1:

 chronos += frametime*4;
  
  if(chronos >= 1.0) injury = 21;

  if(chronos >= 3.0)
  {
  player1.attack = 0;
  chronos = 0.0;
  }
 timer = (int)floor(chronos)+18;

 break;


 case 0:

 chronos += frametime*4;
 if(chronos >= 4.0) chronos = 0.0;
 timer = (int)floor(chronos);
 injury = timer;

 break;
 
 }
}

else                                        // player2 attacking
{
 switch(player2.attack)
 {

 case 4:

  switch(player2.character_offset)
  { 

  case 2:    // LEANDER

  chronos += frametime*2.0;

  if((chronos >= 1.0) && (chronos < 15.0)) 
  {
  chronos += frametime*9.0;
  }

  if(chronos >= 17.0)
  {
  player2.attack = 0;
  chronos = 0.0;
  }

  if((int)floor(chronos) == 0.0)
  {
  timer = 12;
  }

  if((int)floor(chronos) == 1.0 || (int)floor(chronos) == 4.0 || (int)floor(chronos) == 7.0 || 
     (int)floor(chronos) == 10.0 || (int)floor(chronos) == 13.0 ) 
  {
     timer = 13;
     if(injury != 23)
     {
      player2.opponent_energy -= 5;
      injury = 23;
     }
     
  }

  if((int)floor(chronos) == 2.0 || (int)floor(chronos) == 5.0 || (int)floor(chronos) == 8.0 || 
     (int)floor(chronos) == 11.0 || (int)floor(chronos) == 14.0 ) 
  {
     timer = 14;
     if(injury != 22)
     {
      player2.opponent_energy -= 5;
      injury = 22;
     }

  }

  if((int)floor(chronos) == 3.0 || (int)floor(chronos) == 6.0 || (int)floor(chronos) == 9.0 || 
     (int)floor(chronos) == 12.0 || (int)floor(chronos) == 15.0 ) 
  {
     timer = 15;
     injury = 21;
  }

  if(chronos >= 15.0) 
  {
    timer = (int)floor(chronos)+1;
    injury = timer + 7;
  }

  break;

  case 1:    // REBEKHA

  chronos += frametime*2.0;

  if(chronos >= 11.0)
  {
  player2.attack = 0;
  chronos = 0.0;
  }
 
  if((chronos >= 1.0) && (chronos < 10.0)) chronos += frametime*8.0;
 
  if(chronos < 4.0)
  {
  timer = (int)floor(chronos)+12;
   if((chronos >= 1.0) && ((int)floor(chronos) != 2.0) && (injury != timer+8))
   {
   player2.opponent_energy -= 10;
   injury = timer+8;
   }
  }

  if((int)floor(chronos) == 4.0 || (int)floor(chronos) == 6.0 || (int)floor(chronos) == 8.0)
  {
  fallback = 0;
  timer = 14;
  injury = 21;
  }

  if((int)floor(chronos) == 7.0) 
  {
  fallback = 0;
  timer = 15;
   if(injury != 22)
   {
   player2.opponent_energy -= 10;
   injury = 22;
   }
  }

  if((int)floor(chronos) == 5.0 || (int)floor(chronos) == 9.0)
  {
  fallback = 100;
  timer = 16;
   if(injury != 22)
   {
   player2.opponent_energy -= 10;
   injury = 22;
   }
  }

  if(chronos >= 10.0) 
  {
  timer = 17;
  injury = 23;
  }
  
  break;
 
  default:    // CASE 0 - SHIRO

  chronos += frametime*1.0;

  if(chronos >= 3.0)
  {
  chronos += frametime*1.5;
  }

  if(chronos >= 4.0)
  {
  chronos += frametime*1.5;
  }

  if(chronos >= 6.0)
  {
  player2.attack = 0;
  chronos = 0.0;
  }

  timer = (int)floor(chronos)+12;
  if((chronos >= 1.0) && (chronos < 4.0) && (injury != timer+8))
  {
  player2.opponent_energy -= 17;
  injury = timer+8;
  }
  if(chronos >= 4.0)                      injury = 24;
  
  break;

  }

 break;

 case 3:

 chronos += frametime*3.5;
  if(chronos >= 4.0)
  {
  player2.attack = 0;
  chronos = 0.0;
  }
 timer = (int)floor(chronos)+8;
  if((chronos >= 2.0)&&(injury != 23))
  {
  player2.opponent_energy -= 25;
  injury = 23;
  }

 break;


 case 2:

 chronos += frametime*3.3;
  if(chronos >= 4.0)
  {
  player2.attack = 0;
  chronos = 0.0;
  }
 timer = (int)floor(chronos)+4;
  if((injury != timer+17)&&(chronos < 3.0)&&(player2.attack == 2))
  {
  player2.opponent_energy -= 5;
  injury = timer+17;
  }


 break;

 case 1:

 chronos += frametime*4;

  if(chronos >= 1.0) injury = 21;
 
  if(chronos >= 3.0)
  {
  player2.attack = 0;
  chronos = 0.0;
  }

 timer = (int)floor(chronos)+18;


 break;

 case 0:

 chronos += frametime*4;
 if(chronos >= 4.0) chronos = 0.0;
 timer = (int)floor(chronos);
 injury = timer;

 break;
 
 }
}

   if(flash) flash = false;
   else flash = true;
}


/* Main program loop =========================================================*/
int main(int argc, char** argv)
{
   glutInit(&argc, argv);
   printf("\n");

   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);

   if(argc>1 && !strcmp(argv[1], "-f"))
   {
      glutGameModeString("640x480:16@60");
      glutEnterGameMode();
   }
   else
   {
      glutInitWindowSize(640, 480);
      glutInitWindowPosition(100, 100);
      glutCreateWindow("Puzzle Ruckus");
   }

   init();

   /* GLUT callback functions... */
   changeGameMode(TITLES);

   glutReshapeFunc(reshape);
   glutKeyboardFunc(keyDown);
   glutKeyboardUpFunc(keyUp);
   glutSpecialFunc(specialDown);
   glutSpecialUpFunc(specialUp);

   glutMainLoop();

   return 0;
}
