/******************************************************************************/
/*                                                                            */
/*        Project: Puzzle Ruckus                                              */
/*          Title: Tetris functions                                           */
/*        Version: 0.2                                                        */
/*                                                                            */
/*        Started: 10/01/2001                                                 */
/*   Last Updated: 28/01/2001                                                 */
/*                                                                            */
/*         Author: Martin Johnson, Michael Bonnington                         */
/*                                                                            */
/******************************************************************************/

#include "./tetris.h"



bool tetris::possible_blocks[19][4][4] = {
      {{0, 0, 0, 0}, {0, 1, 1, 0}, {0, 1, 1, 0}, {0, 0, 0, 0}},   //block 0

      {{0, 1, 0, 0}, {0, 1, 0, 0}, {0, 1, 0, 0}, {0, 1, 0, 0}},   //block 1
      {{0, 0, 0, 0}, {0, 0, 0, 0}, {1, 1, 1, 1}, {0, 0, 0, 0}},

      {{0, 0, 0, 0}, {0, 1, 0, 0}, {1, 1, 0, 0}, {1, 0, 0, 0}},   //block 2
      {{0, 0, 0, 0}, {0, 0, 0, 0}, {1, 1, 0, 0}, {0, 1, 1, 0}},

      {{0, 0, 0, 0}, {0, 1, 0, 0}, {0, 1, 1, 0}, {0, 0, 1, 0}},   //block 3
      {{0, 0, 0, 0}, {0, 0, 0, 0}, {0, 1, 1, 0}, {1, 1, 0, 0}},   //(mirror of 2)

      {{0, 0, 0, 0}, {0, 1, 0, 0}, {0, 1, 0, 0}, {0, 1, 1, 0}},   //block 4
      {{0, 0, 0, 0}, {0, 0, 1, 0}, {1, 1, 1, 0}, {0, 0, 0, 0}},
      {{0, 0, 0, 0}, {1, 1, 0, 0}, {0, 1, 0, 0}, {0, 1, 0, 0}},
      {{0, 0, 0, 0}, {0, 0, 0, 0}, {1, 1, 1, 0}, {1, 0, 0, 0}},

      {{0, 0, 0, 0}, {0, 1, 0, 0}, {0, 1, 0, 0}, {1, 1, 0, 0}},   //block 5
      {{0, 0, 0, 0}, {0, 0, 0, 0}, {1, 1, 1, 0}, {0, 0, 1, 0}},   //(mirror of 4)
      {{0, 0, 0, 0}, {0, 1, 1, 0}, {0, 1, 0, 0}, {0, 1, 0, 0}},
      {{0, 0, 0, 0}, {1, 0, 0, 0}, {1, 1, 1, 0}, {0, 0, 0, 0}},

      {{0, 0, 0, 0}, {0, 1, 0, 0}, {1, 1, 1, 0}, {0, 0, 0, 0}},   //block 6
      {{0, 0, 0, 0}, {0, 1, 0, 0}, {1, 1, 0, 0}, {0, 1, 0, 0}},
      {{0, 0, 0, 0}, {0, 0, 0, 0}, {1, 1, 1, 0}, {0, 1, 0, 0}},
      {{0, 0, 0, 0}, {0, 1, 0, 0}, {0, 1, 1, 0}, {0, 1, 0, 0}}};



/* Reset =====================================================================*/
void tetris::reset(int initial_level)
{
   score = 0; lines = 0; level = initial_level;
   opponent_energy = 100; power = 0; attack = 0;
   display_next_block = true;
   game_over = false;
   pos_x = 4, pos_y = 0, drop = 0;

   clearGrid();
   initNewBlock(randomBlock());
}

/* Clear grid ================================================================*/
void tetris::clearGrid(void)
{
   int x, y;

   for(y=0; y<25; y++)
   {
      for(x=0; x<12; x++)
      {
         if(x>0 && x<11 && x>=0 && y<24) grid[x][y] = false;
         else grid[x][y] = true;
      }
   }
}

/* Initialise new block ======================================================*/
void tetris::initNewBlock(int block)
{
   pos_x = 4; pos_y = 0, drop = 0;

   if(!checkCollision(0))
   {
      changeBlock(block);
      next_block_type = randomBlock();
   }
   else game_over = true;
}

/* Change block array ========================================================*/
void tetris::changeBlock(int block)
{
   int x, y;

   for(y=0; y<4; y++)
   {
      for(x=0; x<4; x++)
      {
         current_block_type = block;
         current_block[x][y] = possible_blocks[block][y][x];
      }
   }
}

/* Create temporary block array ==============================================*/
void tetris::tempBlock(int block)
{
   int x, y;

   for(y=0; y<4; y++)
   {
      for(x=0; x<4; x++)
      {
         temp_block[x][y] = possible_blocks[block][y][x];
      }
   }
}

/* Move block ================================================================*/
void tetris::moveBlock(int direction)
{
   if(!checkCollision(direction))
   {
      pos_x += direction;
      //pos_x += frametime*12;
   }
}

/* Drop block ================================================================*/
void tetris::dropBlock(double frametime)
{
   int start_y;

   start_y = (int)drop;
   drop += frametime*24;
   score += drop-start_y;

   if(power < 100) power += (drop-start_y)/2; //*(level/2);
   else power = 100;
}

/* Rotate block ==============================================================*/
void tetris::rotateBlock(bool clockwise)
{
   switch(current_block_type)
   {
      case 0:
         //do nothing
         break;

      case 1:
         tempBlock(2);
         if(!checkRotate()) changeBlock(2);
         break;

      case 2:
         tempBlock(1);
         if(!checkRotate()) changeBlock(1);
         break;

      case 3:
         tempBlock(4);
         if(!checkRotate()) changeBlock(4);
         break;

      case 4:
         tempBlock(3);
         if(!checkRotate()) changeBlock(3);
         break;

      case 5:
         tempBlock(6);
         if(!checkRotate()) changeBlock(6);
         break;

      case 6:
         tempBlock(5);
         if(!checkRotate()) changeBlock(5);
         break;

      case 7:
         if(clockwise)
         {
            tempBlock(10);
            if(!checkRotate()) changeBlock(10);
         }
         else
         {
            tempBlock(8);
            if(!checkRotate()) changeBlock(8);
         }
         break;

      case 8:
         if(clockwise)
         {
            tempBlock(7);
            if(!checkRotate()) changeBlock(7);
         }
         else
         {
            tempBlock(9);
            if(!checkRotate()) changeBlock(9);
         }
         break;

      case 9:
         if(clockwise)
         {
            tempBlock(8);
            if(!checkRotate()) changeBlock(8);
         }
         else
         {
            tempBlock(10);
            if(!checkRotate()) changeBlock(10);
         }
         break;

      case 10:
         if(clockwise)
         {
            tempBlock(9);
            if(!checkRotate()) changeBlock(9);
         }
         else
         {
            tempBlock(7);
            if(!checkRotate()) changeBlock(7);
         }
         break;

      case 11:
         if(clockwise)
         {
            tempBlock(14);
            if(!checkRotate()) changeBlock(14);
         }
         else
         {
            tempBlock(12);
            if(!checkRotate()) changeBlock(12);
         }
         break;

      case 12:
         if(clockwise)
         {
            tempBlock(11);
            if(!checkRotate()) changeBlock(11);
         }
         else
         {
            tempBlock(13);
            if(!checkRotate()) changeBlock(13);
         }
         break;

      case 13:
         if(clockwise)
         {
            tempBlock(12);
            if(!checkRotate()) changeBlock(12);
         }
         else
         {
            tempBlock(14);
            if(!checkRotate()) changeBlock(14);
         }
         break;

      case 14:
         if(clockwise)
         {
            tempBlock(13);
            if(!checkRotate()) changeBlock(13);
         }
         else
         {
            tempBlock(11);
            if(!checkRotate()) changeBlock(11);
         }
         break;

      case 15:
         if(clockwise)
         {
            tempBlock(18);
            if(!checkRotate()) changeBlock(18);
         }
         else
         {
            tempBlock(16);
            if(!checkRotate()) changeBlock(16);
         }
         break;

      case 16:
         if(clockwise)
         {
            tempBlock(15);
            if(!checkRotate()) changeBlock(15);
         }
         else
         {
            tempBlock(17);
            if(!checkRotate()) changeBlock(17);
         }
         break;

      case 17:
         if(clockwise)
         {
            tempBlock(16);
            if(!checkRotate()) changeBlock(16);
         }
         else
         {
            tempBlock(18);
            if(!checkRotate()) changeBlock(18);
         }
         break;

      case 18:
         if(clockwise)
         {
            tempBlock(17);
            if(!checkRotate()) changeBlock(17);
         }
         else
         {
            tempBlock(15);
            if(!checkRotate()) changeBlock(15);
         }
         break;

      default:
         break;
   }
}

/* Check for collision =======================================================*/
bool tetris::checkCollision(int block_direction)
{
   int x, y, vertical = 0;
   bool collide = false;

   if(block_direction == 0) vertical = 1;

   for(y=0; y<4; y++)
   {
      for(x=0; x<4; x++)
      {
         if(current_block[x][y] &&
            grid[x+(int)pos_x+block_direction][y+(int)pos_y+vertical])
         {
            collide = true;
            x=4; y=4;
         }
      }
   }

   return collide;
}

/* Check for collisions when rotating blocks =================================*/
bool tetris::checkRotate(void)
{
   int x, y;
   bool collide = false;

   for(y=0; y<4; y++)
   {
      for(x=0; x<4; x++)
      {
         if(temp_block[x][y] && grid[x+(int)pos_x][y+(int)pos_y])
         {
            collide = true;
            x=4; y=4;
         }
      }
   }

   return collide;
}

/* Check for lines ===========================================================*/
void tetris::checkLines(void)
{
   int x, y, height, line_counter = 0;

   for(y=0; y<24; y++)
   {
      if(grid[1][y] && grid[2][y] && grid[3][y] && grid[4][y] && grid[5][y] &&
         grid[6][y] && grid[7][y] && grid[8][y] && grid[9][y] && grid[10][y])
      {
         for(x=1; x<11; x++) grid[x][y] = false;
         height = y; line_counter++; lines++;
         if(power >=90) power = 0;

         if(level <= lines/10) level = lines/10;

         dropGrid(height);
      }
   }

   switch(line_counter)
   {
      case 1:
         attack = 1;
         score += 40*(level+1);
         opponent_energy -= 5;
         break;

      case 2:
         attack = 2;
         score += 100*(level+1);
         break;

      case 3:
         attack = 3;
         score += 300*(level+1);
         break;

      case 4:
         attack = 4;
         score += 1200*(level+1);
         break;

      default:
         break;
   }
}

/* Drop down blocks above completed lines ====================================*/
void tetris::dropGrid(int height)
{
   int x, y;

   for(y=height; y>0; y--)
   {
      for(x=1; x<11; x++)
      {
         grid[x][y] = grid[x][y-1];
      }
   }
}

/* Set block =================================================================*/
void tetris::setBlock(void)
{
   int x, y;

   for(y=0; y<4; y++)
   {
      for(x=0; x<4; x++)
      {
         if(current_block[x][y]) grid[x+(int)pos_x][y+(int)pos_y] = true;
      }
   }
}

/* Return random integer =====================================================*/
int tetris::randomBlock(void)
{
   int random_block_type, value;

   random_block_type = rand() % 7;

   switch(random_block_type)
   {
      case 0:
         value = 0;
         break;

      case 1:
         value = (rand() % 2)+1;
         break;

      case 2:
         value = (rand() % 2)+3;
         break;

      case 3:
         value = (rand() % 2)+5;
         break;

      case 4:
         value = (rand() % 4)+7;
         break;

      case 5:
         value = (rand() % 4)+11;
         break;

      case 6:
         value = (rand() % 4)+15;
         break;

      default:
         break;
   }

   return value;
}

/* Update everything =========================================================*/
bool tetris::update(bool keypress, double frametime)
{
   drop += frametime*((level/2)+1);

   if(!keypress && power > 0) power -= frametime*4; //*(level/4);

   /* Drop blocks automatically until a collision occurs... */
   if(!checkCollision(0))
   {
      pos_y = drop;
      return false;
   }
   else
   {
      if(keypress)
      {
         setBlock();
         checkLines();
         initNewBlock(next_block_type);
      }
      else if(drop > pos_y+1)
      {
         setBlock();
         checkLines();
         initNewBlock(next_block_type);
      }
      return true;
   }
}
