/******************************************************************************/
/*                                                                            */
/*        Project: Boona Taxi                                                 */
/*          Title: Frame counter for WIN32                                    */
/*        Version: 0.31                                                       */
/*                                                                            */
/*        Started: 20/09/2000                                                 */
/*   Last Updated: 19/11/2000                                                 */
/*                                                                            */
/*        Authors: Michael Bonnington                                         */
/*                                                                            */
/******************************************************************************/

#include <stdio.h>
#include "./win32.h"
#include "./frametime.h"



/* Globals ===================================================================*/
double framestart, frameend, frametime;
__int64 startclock;
double rate_inv;



/* Frame counter - initialise ================================================*/
int initFrameCounter(void)
{
   __int64 rate;
   if(!QueryPerformanceFrequency((LARGE_INTEGER*)&rate))
      return 0;

   /* usually the rate will be 1193180... */
   if(!rate)
      return 0;

   rate_inv=1.0/(double)rate;
   if(!QueryPerformanceCounter((LARGE_INTEGER*)&startclock))
      return 0;

   printf("Found hi-res clock.\n\n");
   return 1;   /* There is a clock */
}

/* Frame counter - get the timer's value and divide by its rate ==============*/
double ftime(void)
{
   __int64 endclock;
   QueryPerformanceCounter((LARGE_INTEGER*)&endclock);
   return (double)(endclock-startclock)*rate_inv;
}

/* Frame counter - work out current frametime ================================*/
double getFrametime(void)
{
   do{
      frameend=ftime();
   }while(frameend==framestart);

   frametime=frameend-framestart;
   framestart=frameend;

   return frametime;
}
