/******************************************************************************/
/*                                                                            */
/*        Project: GL Tetris                                                  */
/*          Title:                                                            */
/*        Version: 0.1                                                        */
/*                                                                            */
/*        Started: 10/01/2001                                                 */
/*   Last Updated: 13/01/2001                                                 */
/*                                                                            */
/*         Author: Michael Bonnington                                         */
/*                                                                            */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "./win32.h"
#include "./font.h"
#include "./frametime.h"



/* Constants =================================================================*/
#define TITLES 1
#define GAME 2
#define PAUSED 3



/* Function prototypes =======================================================*/
void init(void);
void changeGameMode(int);

void initTetris(void);
void initNewBlock(int);
void changeBlock(int);
void tempBlock(int);
void rotateBlock(bool);
bool checkCollision(int);
bool checkRotate(void);
void checkLines(void);
void dropBlocks(int);
void setBlock(void);
int randomBlock(void);

void drawGrid(int, int, bool);
void drawBlock(int, int, bool);
void drawStatus(void);

/* GLUT callback functions... */
void idleTitles(void);
void displayTitles(void);
void idleGame(void);
void displayGame(void);
void idlePaused(void);
void displayPaused(void);
void reshape(int, int);
void keyDown(unsigned char, int, int);
void keyUp(unsigned char, int, int);
void specialDown(int, int, int);
void specialUp(int, int, int);



/* Structure definitions =====================================================*/

/* Class definitions =========================================================*/

/* Globals ===================================================================*/
int win_width, win_height;
int gameMode;
bool keys[128], specials[128];

int score = 0, lines = 0, level = 0;
char scoreText[10], linesText[10], levelText[10];
bool display_next_block = true;
float pos_x = 4, pos_y = 0, drop = 0;   //position of block

int current_block_type;
int next_block_type;

bool grid[12][25];
bool temp_grid[12][25];
bool current_block[4][4];
bool temp_block[4][4];
bool possible_blocks[19][4][4] = {
{{0, 0, 0, 0}, {0, 1, 1, 0}, {0, 1, 1, 0}, {0, 0, 0, 0}},   //block 0

{{0, 1, 0, 0}, {0, 1, 0, 0}, {0, 1, 0, 0}, {0, 1, 0, 0}},   //block 1
{{0, 0, 0, 0}, {0, 0, 0, 0}, {1, 1, 1, 1}, {0, 0, 0, 0}},

{{0, 0, 0, 0}, {0, 1, 0, 0}, {1, 1, 0, 0}, {1, 0, 0, 0}},   //block 2
{{0, 0, 0, 0}, {0, 0, 0, 0}, {1, 1, 0, 0}, {0, 1, 1, 0}},

{{0, 0, 0, 0}, {0, 1, 0, 0}, {0, 1, 1, 0}, {0, 0, 1, 0}},   //block 3
{{0, 0, 0, 0}, {0, 0, 0, 0}, {0, 1, 1, 0}, {1, 1, 0, 0}},   //(mirror of 2)

{{0, 0, 0, 0}, {0, 1, 0, 0}, {0, 1, 0, 0}, {0, 1, 1, 0}},   //block 4
{{0, 0, 0, 0}, {0, 0, 1, 0}, {1, 1, 1, 0}, {0, 0, 0, 0}},
{{0, 0, 0, 0}, {1, 1, 0, 0}, {0, 1, 0, 0}, {0, 1, 0, 0}},
{{0, 0, 0, 0}, {0, 0, 0, 0}, {1, 1, 1, 0}, {1, 0, 0, 0}},

{{0, 0, 0, 0}, {0, 1, 0, 0}, {0, 1, 0, 0}, {1, 1, 0, 0}},   //block 5
{{0, 0, 0, 0}, {0, 0, 0, 0}, {1, 1, 1, 0}, {0, 0, 1, 0}},   //(mirror of 4)
{{0, 0, 0, 0}, {0, 1, 1, 0}, {0, 1, 0, 0}, {0, 1, 0, 0}},
{{0, 0, 0, 0}, {1, 0, 0, 0}, {1, 1, 1, 0}, {0, 0, 0, 0}},

{{0, 0, 0, 0}, {0, 1, 0, 0}, {1, 1, 1, 0}, {0, 0, 0, 0}},   //block 6
{{0, 0, 0, 0}, {0, 1, 0, 0}, {1, 1, 0, 0}, {0, 1, 0, 0}},
{{0, 0, 0, 0}, {0, 0, 0, 0}, {1, 1, 1, 0}, {0, 1, 0, 0}},
{{0, 0, 0, 0}, {0, 1, 0, 0}, {0, 1, 1, 0}, {0, 1, 0, 0}}};



/* Functions =================================================================*/

/* Initialisation function ===================================================*/
void init(void)
{
   /* Initialise OpenGL/GLUT stuff... */
   glClearColor(0.5, 0.5, 0.5, 0.0);
   glShadeModel(GL_FLAT);
   glEnable(GL_CULL_FACE);
   glCullFace(GL_BACK);

   glutIgnoreKeyRepeat(1);
   glutSetCursor(GLUT_CURSOR_NONE);

   makeRasterFont();   //initialise font

   /* Initialise frame time function... */
   if(!initFrameCounter())
   {
      printf("ERROR: Unable to initialise frame counter.\n\n");
      exit(0);
   }

   initTetris();   //initialise Tetris
}

/* Function to change current game mode ======================================*/
void changeGameMode(int mode)
{
   switch(mode)
   {
      case TITLES:
         display_next_block = false;
         gameMode = TITLES;
         glutIdleFunc(idleTitles);
         glutDisplayFunc(displayTitles);
         break;

      case GAME:
         display_next_block = true;
         gameMode = GAME;
         glutIdleFunc(idleGame);
         glutDisplayFunc(displayGame);
         break;

      case PAUSED:
         display_next_block = false;
         gameMode = PAUSED;
         glutIdleFunc(idlePaused);
         glutDisplayFunc(displayPaused);
         break;

      default:
         break;
   }
}



/* Initialise new game =======================================================*/
void initTetris(void)
{
   int x, y;

   score = 0; lines = 0;

   for(y=0; y<25; y++)
   {
      for(x=0; x<12; x++)
      {
         if(x>0 && x<11 && x>=0 && y<24) grid[x][y] = false;
         else grid[x][y] = true;
      }
   }

   initNewBlock(randomBlock());
}

/* Initialise new block ======================================================*/
void initNewBlock(int block)
{
   pos_x = 4; pos_y = 0, drop = 0;

   if(!checkCollision(0))
   {
      changeBlock(block);
      next_block_type = randomBlock();
   }
   else changeGameMode(TITLES);
}

/* Change block array ========================================================*/
void changeBlock(int block)
{
   int x, y;

   for(y=0; y<4; y++)
   {
      for(x=0; x<4; x++)
      {
         current_block_type = block;
         current_block[x][y] = possible_blocks[block][y][x];
      }
   }
}

/* Create temporary block array ==============================================*/
void tempBlock(int block)
{
   int x, y;

   for(y=0; y<4; y++)
   {
      for(x=0; x<4; x++)
      {
         temp_block[x][y] = possible_blocks[block][y][x];
      }
   }
}

/* Rotate block ==============================================================*/
void rotateBlock(bool clockwise)
{
   switch(current_block_type)
   {
      case 0:
         //do nothing
         break;

      case 1:
         tempBlock(2);
         if(!checkRotate()) changeBlock(2);
         break;

      case 2:
         tempBlock(1);
         if(!checkRotate()) changeBlock(1);
         break;

      case 3:
         tempBlock(4);
         if(!checkRotate()) changeBlock(4);
         break;

      case 4:
         tempBlock(3);
         if(!checkRotate()) changeBlock(3);
         break;

      case 5:
         tempBlock(6);
         if(!checkRotate()) changeBlock(6);
         break;

      case 6:
         tempBlock(5);
         if(!checkRotate()) changeBlock(5);
         break;

      case 7:
         if(clockwise)
         {
            tempBlock(10);
            if(!checkRotate()) changeBlock(10);
         }
         else
         {
            tempBlock(8);
            if(!checkRotate()) changeBlock(8);
         }
         break;

      case 8:
         if(clockwise)
         {
            tempBlock(7);
            if(!checkRotate()) changeBlock(7);
         }
         else
         {
            tempBlock(9);
            if(!checkRotate()) changeBlock(9);
         }
         break;

      case 9:
         if(clockwise)
         {
            tempBlock(8);
            if(!checkRotate()) changeBlock(8);
         }
         else
         {
            tempBlock(10);
            if(!checkRotate()) changeBlock(10);
         }
         break;

      case 10:
         if(clockwise)
         {
            tempBlock(9);
            if(!checkRotate()) changeBlock(9);
         }
         else
         {
            tempBlock(7);
            if(!checkRotate()) changeBlock(7);
         }
         break;

      case 11:
         if(clockwise)
         {
            tempBlock(14);
            if(!checkRotate()) changeBlock(14);
         }
         else
         {
            tempBlock(12);
            if(!checkRotate()) changeBlock(12);
         }
         break;

      case 12:
         if(clockwise)
         {
            tempBlock(11);
            if(!checkRotate()) changeBlock(11);
         }
         else
         {
            tempBlock(13);
            if(!checkRotate()) changeBlock(13);
         }
         break;

      case 13:
         if(clockwise)
         {
            tempBlock(12);
            if(!checkRotate()) changeBlock(12);
         }
         else
         {
            tempBlock(14);
            if(!checkRotate()) changeBlock(14);
         }
         break;

      case 14:
         if(clockwise)
         {
            tempBlock(13);
            if(!checkRotate()) changeBlock(13);
         }
         else
         {
            tempBlock(11);
            if(!checkRotate()) changeBlock(11);
         }
         break;

      case 15:
         if(clockwise)
         {
            tempBlock(18);
            if(!checkRotate()) changeBlock(18);
         }
         else
         {
            tempBlock(16);
            if(!checkRotate()) changeBlock(16);
         }
         break;

      case 16:
         if(clockwise)
         {
            tempBlock(15);
            if(!checkRotate()) changeBlock(15);
         }
         else
         {
            tempBlock(17);
            if(!checkRotate()) changeBlock(17);
         }
         break;

      case 17:
         if(clockwise)
         {
            tempBlock(16);
            if(!checkRotate()) changeBlock(16);
         }
         else
         {
            tempBlock(18);
            if(!checkRotate()) changeBlock(18);
         }
         break;

      case 18:
         if(clockwise)
         {
            tempBlock(17);
            if(!checkRotate()) changeBlock(17);
         }
         else
         {
            tempBlock(15);
            if(!checkRotate()) changeBlock(15);
         }
         break;

      default:
         break;
   }
}

/* Check for collision =======================================================*/
bool checkCollision(int block_direction)
{
   int x, y, vertical = 0;
   bool collide = false;

   if(block_direction == 0) vertical = 1;

   for(y=0; y<4; y++)
   {
      for(x=0; x<4; x++)
      {
         if(current_block[x][y] &&
            grid[x+(int)pos_x+block_direction][y+(int)pos_y+vertical])
         {
            collide = true;
            x=4; y=4;
         }
      }
   }

   return collide;
}

/* Check for collisions when rotating blocks =================================*/
bool checkRotate(void)
{
   int x, y;
   bool collide = false;

   for(y=0; y<4; y++)
   {
      for(x=0; x<4; x++)
      {
         if(temp_block[x][y] && grid[x+(int)pos_x][y+(int)pos_y])
         {
            collide = true;
            x=4; y=4;
         }
      }
   }

   return collide;
}

/* Check for lines ===========================================================*/
void checkLines(void)
{
   int x, y, height, line_counter = 0;

   for(y=0; y<24; y++)
   {
      if(grid[1][y] && grid[2][y] && grid[3][y] && grid[4][y] && grid[5][y] &&
         grid[6][y] && grid[7][y] && grid[8][y] && grid[9][y] && grid[10][y])
      {
         for(x=1; x<11; x++) grid[x][y] = false;
         height = y; line_counter++; lines++;

         if(level <= lines/10) level = lines/10;

         dropBlocks(height);
      }
   }

   switch(line_counter)
   {
      case 1:
         score += 40*(level+1);
         break;

      case 2:
         score += 100*(level+1);
         break;

      case 3:
         score += 300*(level+1);
         break;

      case 4:
         score += 1200*(level+1);
         break;

      default:
         break;
   }
}

/* Drop down blocks above completed lines ====================================*/
void dropBlocks(int height)
{
   int x, y;

   for(y=height; y>0; y--)
   {
      for(x=1; x<11; x++)
      {
         grid[x][y] = grid[x][y-1];
      }
   }
}

/* Set block =================================================================*/
void setBlock(void)
{
   int x, y;

   for(y=0; y<4; y++)
   {
      for(x=0; x<4; x++)
      {
         if(current_block[x][y]) grid[x+(int)pos_x][y+(int)pos_y] = true;
      }
   }
}

/* Return random integer =====================================================*/
int randomBlock(void)
{
   int random_block_type, value;

   random_block_type = rand() % 7;

   switch(random_block_type)
   {
      case 0:
         value = 0;
         break;

      case 1:
         value = (rand() % 2)+1;
         break;

      case 2:
         value = (rand() % 2)+3;
         break;

      case 3:
         value = (rand() % 2)+5;
         break;

      case 4:
         value = (rand() % 4)+7;
         break;

      case 5:
         value = (rand() % 4)+11;
         break;

      case 6:
         value = (rand() % 4)+15;
         break;

      default:
         break;
   }

   return value;
}



/* Draw grid =================================================================*/
void drawGrid(int x, int y, bool solid)
{
   glPushMatrix();
    if(solid)
    {
       glColor3f(1.0, 1.0, 1.0);
       glBegin(GL_QUADS);
        glVertex2d(x-8, y-8);
        glVertex2d(x-8, y+8);
        glVertex2d(x+8, y+8);
        glVertex2d(x+8, y-8);
       glEnd();
       glColor3f(0.0, 0.0, 0.0);
    }
    else
    {
       glColor3f(0.6, 0.6, 0.6);
    }
    glBegin(GL_LINE_LOOP);
     glVertex2d(x-7, y-7);
     glVertex2d(x-7, y+8);
     glVertex2d(x+8, y+8);
     glVertex2d(x+8, y-7);
    glEnd();
   glPopMatrix();
}

/* Draw block ================================================================*/
void drawBlock(int x, int y, bool solid)
{
   glPushMatrix();
    if(solid)
    {
       glColor3f(0.0, 0.0, 0.0);
       glBegin(GL_QUADS);
        glVertex2d(x-8, y-8);
        glVertex2d(x-8, y+8);
        glVertex2d(x+8, y+8);
        glVertex2d(x+8, y-8);
       glEnd();
       glColor3f(1.0, 0.8, 0.2);
       glBegin(GL_QUADS);
        glVertex2d(x-7, y-7);
        glVertex2d(x-7, y+7);
        glVertex2d(x+7, y+7);
        glVertex2d(x+7, y-7);
       glEnd();
    }
   glPopMatrix();
}

/* Draw status display =======================================================*/
void drawStatus(void)
{
   int x, y;
   bool next_block[4][4];

   font("Score", 16, win_height-44, 1.0, 1.0, 1.0);
   font(scoreText, 16, win_height-32, 0.0, 0.0, 0.0);

   font("Lines", 64, win_height-44, 1.0, 1.0, 1.0);
   font(linesText, 64, win_height-32, 0.0, 0.0, 0.0);

   font("Level", 104, win_height-44, 1.0, 1.0, 1.0);
   font(levelText, 104, win_height-32, 0.0, 0.0, 0.0);

   font("Next", 144, win_height-44, 1.0, 1.0, 1.0);
   if(display_next_block)
   {
      for(y=0; y<4; y++)
      {
         for(x=0; x<4; x++)
         {
            next_block[x][y] = possible_blocks[next_block_type][y][x];

            if(next_block[x][y])
            {
               glPushMatrix();
               glTranslated(148, win_height-40, 0);
                glColor3f(0.0, 0.0, 0.0);
                glBegin(GL_QUADS);
                 glVertex2d((x*8)-4, (y*8)-4);
                 glVertex2d((x*8)-4, (y*8)+3);
                 glVertex2d((x*8)+3, (y*8)+3);
                 glVertex2d((x*8)+3, (y*8)-4);
                glEnd();
               glPopMatrix();
            }
         }
      }
   }
}



/* Idle function (Titles) ====================================================*/
void idleTitles(void)
{
   double frametime = getFrametime();

   if(specials[GLUT_KEY_LEFT])   //decrease level
   {
      if(level > 0) level--;
      else level = 9;
      specials[GLUT_KEY_LEFT] = false;
   }

   if(specials[GLUT_KEY_RIGHT])   //increase level
   {
      if(level < 9) level++;
      else level = 0;
      specials[GLUT_KEY_RIGHT] = false;
   }

   if(keys[27])   //quit
   {
      exit(0);
      keys[27] = false;
   }

   if(keys[13] || specials[GLUT_KEY_F2])   //begin game
   {
      changeGameMode(GAME);
      initTetris();
      keys[13] = false;
      specials[GLUT_KEY_F2] = false;
   }

   sprintf(scoreText, "%d", score);
   sprintf(linesText, "%d", lines);
   sprintf(levelText, "%d", level);

   glutPostRedisplay();
}

/* Display function (Titles) =================================================*/
void displayTitles(void)
{
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   glLoadIdentity();

   font("Tetris", 16, 28, 0.0, 0.0, 0.0);
   font("Game Over", 16, 180, 1.0, 1.0, 1.0);
   font("Select a level with LEFT or RIGHT.", 16, 192, 1.0, 1.0, 1.0);
   font("ENTER or F2 starts a new game.", 16, 204, 1.0, 1.0, 1.0);
   font("Level", 16, 228, 1.0, 1.0, 1.0);
   font(levelText, 16, 240, 0.0, 0.0, 0.0);
   drawStatus();   

   glutSwapBuffers();
}

/* Idle function (Game) ======================================================*/
void idleGame(void)
{
   double frametime = getFrametime();
   int start_y;

   srand(time(NULL));

   if(specials[GLUT_KEY_LEFT])   //move left
   {
      if(!checkCollision(-1))
      {
         pos_x--;
         //pos_x -= frametime*12;
      }
      specials[GLUT_KEY_LEFT] = false;
   }

   if(specials[GLUT_KEY_RIGHT])   //move right
   {
      if(!checkCollision(1))
      {
         pos_x++;
         //pos_x += frametime*12;
      }
      specials[GLUT_KEY_RIGHT] = false;
   }

   if(specials[GLUT_KEY_DOWN])   //drop
   {
      start_y = (int)drop;
      drop += frametime*24;
      score += drop-start_y;
   }

   if(keys['z'] || keys['Z'] || specials[GLUT_KEY_UP])   //rotate anti-clockwise
   {
      rotateBlock(0);
      keys['z'] = false;
      keys['Z'] = false;
      specials[GLUT_KEY_UP] = false;
   }

   if(keys['x'] || keys['X'])   //rotate clockwise
   {
      rotateBlock(1);
      keys['x'] = false;
      keys['X'] = false;
   }

   if(keys[27])   //go back to titles
   {
      changeGameMode(TITLES);
      keys[27] = false;
   }

   if(keys['p'] || keys['P'])   //pause
   {
      changeGameMode(PAUSED);
      keys['p'] = false;
      keys['P'] = false;
   }

   if(specials[GLUT_KEY_F2])   //reset
   {
      initTetris();
      specials[GLUT_KEY_F2] = false;
   }

   if(specials[GLUT_KEY_F3])   //whether to show or hide next block
   {
      if(display_next_block) display_next_block = false;
      else display_next_block = true;
      specials[GLUT_KEY_F3] = false;
   }

   drop += frametime*((level/2)+1);

   /* Drop blocks automatically until a collision occurs... */
   if(!checkCollision(0)) pos_y = drop;
   else
   {
      if(specials[GLUT_KEY_DOWN])
      {
         setBlock();
         checkLines();
         initNewBlock(next_block_type);
      }
      else if(drop > pos_y+1)
      {
         setBlock();
         checkLines();
         initNewBlock(next_block_type);
      }
      specials[GLUT_KEY_LEFT] = false;
      specials[GLUT_KEY_RIGHT] = false;
      specials[GLUT_KEY_DOWN] = false;
   }

   sprintf(scoreText, "%d", score);
   sprintf(linesText, "%d", lines);
   sprintf(levelText, "%d", level);

   glutPostRedisplay();
}

/* Display function (Game) ===================================================*/
void displayGame(void)
{
   int x, y;

   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   glLoadIdentity();

   glPushMatrix();
    glTranslated(8, 32, 0);

    /* Draw grid... */
    for(y=0; y<24; y++)
    {
       for(x=1; x<11; x++)
       {
          drawGrid(x*16, y*16, grid[x][y]);
       }
    }

    /* Draw current block... */
    for(y=0; y<4; y++)
    {
       for(x=0; x<4; x++)
       {
          drawBlock((x+(int)pos_x)*16, (y+(int)pos_y)*16, current_block[x][y]);
       }
    }
   glPopMatrix();

   drawStatus();   

   glutSwapBuffers();
}

/* Idle function (Paused) ====================================================*/
void idlePaused(void)
{
   double frametime = getFrametime();

   if(keys[27])   //go back to titles
   {
      changeGameMode(TITLES);
      keys[27] = false;
   }

   if(keys['p'] || keys['P'])   //unpause
   {
      changeGameMode(GAME);
      keys['p'] = false;
      keys['P'] = false;
   }

   if(specials[GLUT_KEY_F2])   //reset
   {
      changeGameMode(GAME);
      initTetris();
      specials[GLUT_KEY_F2] = false;
   }

   if(specials[GLUT_KEY_F3])   //whether to show or hide next block
   {
      if(display_next_block) display_next_block = false;
      else display_next_block = true;
      specials[GLUT_KEY_F3] = false;
   }

   glutPostRedisplay();
}

/* Display function (Paused) =================================================*/
void displayPaused(void)
{
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   glLoadIdentity();

   font("Game Paused - press P to resume", 16, 28, 1.0, 1.0, 1.0);
   drawStatus();   

   glutSwapBuffers();
}

/* Reshape function ==========================================================*/
void reshape(int w, int h)
{
   if(h == 0) h = 1;   //required to prevent crash when window is minimised
   win_width = w; win_height = h;

   glViewport(0, 0, (GLsizei) w, (GLsizei) h);

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glOrtho(0, w, h, 0, -1, 1);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
}

/* Keyboard function - key is pressed ========================================*/
void keyDown(unsigned char key, int x, int y)
{
   keys[key] = true;
}

/* Keyboard function - key is released =======================================*/
void keyUp(unsigned char key, int x, int y)
{
   keys[key] = false;
}

/* Keyboard function - special key is pressed ================================*/
void specialDown(int key, int x, int y)
{
   specials[key] = true;
}

/* Keyboard function - special key is released ===============================*/
void specialUp(int key, int x, int y)
{
   specials[key] = false;
}



/* Main program loop =========================================================*/
int main(int argc, char** argv)
{
   glutInit(&argc, argv);
   printf("\n");

   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);

   if(argc>1 && !strcmp(argv[1], "-f"))
   {
      glutGameModeString("640x480:16@60");
      glutEnterGameMode();
   }
   else
   {
      glutInitWindowSize(192, 480);
      glutInitWindowPosition(100, 100);
      glutCreateWindow("Tetris");
   }

   init();

   /* GLUT callback functions... */
   changeGameMode(TITLES);

   glutReshapeFunc(reshape);
   glutKeyboardFunc(keyDown);
   glutKeyboardUpFunc(keyUp);
   glutSpecialFunc(specialDown);
   glutSpecialUpFunc(specialUp);

   glutMainLoop();

   return 0;
}
